#!/usr/bin/python
# -*- coding: utf-8 -*-
import logging
from collections import namedtuple
Result = namedtuple("Result", "sql params")

logger = logging.getLogger(__name__)

class SQLFactory(object):
    """
    Generic SQL Builder
    """
    def __init__(self, tablename=None, indicators=None, filters=None,
                 start_date=None, end_date=None):
        self._tablename = tablename if tablename else "eval_virtual_participations"
        self._indicators = indicators if indicators else []
        self._filters = filters if filters else []
        self._groupby = ["group by gender"]
        self._additional_selects = []
        self.start_date = start_date
        self.end_date = end_date
        logger.debug("Start SQLFactory with Table %s" % self._tablename)

    @property
    def indicators(self):
        return self._indicators

    @indicators.setter
    def indicators(self, indicators):
        self._indicators = indicators
        logger.debug("set_indicators to %s" % str(self._indicators))

    @property
    def filters(self):
        return self._filters

    @filters.setter
    def filters(self, filters):
        self._filters = filters
        logger.debug("set_filters to %s" % str(self._filters))

    @property
    def group_by(self):
        return self._groupby

    @group_by.setter
    def group_by(self, value):
        self._groupby = value

    @property
    def additional_selects(self):
        return self._additional_selects

    @additional_selects.setter
    def additional_selects(self, value):
        self._additional_selects = value

    def build_where_clause(self):
        """
        Builds where-clause from filter conditions
        :return: where-clause as string and parameters
        """
        conditions = [condition[0] for condition in self._filters]
        where_clause = "WHERE %s" % (" AND ".join(conditions))
        return where_clause

    def build_sum_column(self, indicator):
        return "SUM(CASE WHEN %s = TRUE THEN 1 ELSE 0 END) AS %s" % (indicator, indicator)

    def create(self):
        """
        Creates SQL on the fly
        :return: Result as named tuple (sql for querystring and params for parameters)
        """
        self.check_for_indicators()
        select = ["SELECT MAX(CASE WHEN gender=0 THEN 'female'"
                  " ELSE 'male' END) as gender",
                  "COUNT(gender) as total",
                  "SUM(CASE WHEN start_date is not null "
                  " and start_date >= %s THEN 1 ELSE 0 END) AS enters",
                  "SUM(CASE WHEN (start_date is not null "
                  " and end_date is not null and end_date <= %s"
                  " ) THEN 1 ELSE 0 END)"
                  " AS exits"]
        select.extend(self._additional_selects)
        indicator_columns = [self.build_sum_column(indicator) for indicator in self._indicators]
        select_clause = [", ".join(select + indicator_columns)]
        from_clause = ["FROM %s" % self._tablename]
        group_by = self._groupby
        where_clause = []
        if self.filters:
            where_clause.append(self.build_where_clause())
        query = " ".join(select_clause + from_clause + where_clause + group_by)
        params = [parameter[1] for parameter in self._filters if parameter[1]]
        params.insert(0, self.end_date)
        params.insert(0, self.start_date)
        result = Result(query, params)
        logger.debug("create SQLCMD '%s' with params '%s'" % result)
        return result

    def check_for_indicators(self):
        if not self._indicators:
            raise ValueError("Missing indicators")
